/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 * This code was modified by Silvio Evangelista e Kleber Sampaio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
//package hypertree;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
//import javax.swing.*;
import java.applet.*;
import java.io.*;

/**
 * The HTAction class manage the action on the hypertree :
 * drag of a node...
 *
 * @author     silvio
 * @created    29 de Setembro de 2004
 */

/*
 *  Antes:
 *  class HTAction
 *  extends MouseAdapter
 *  implements MouseMotionListener  {
 */
class HTAction
//extends MouseAdapter
		 implements Serializable, MouseMotionListener, MouseListener, KeyListener, ActionListener {

	private HTDraw model = null;
	// the drawing model
	private HTNodeLabel label = null;

	private HTCoordE startPoint = null;
	// starting point of dragging
	private HTCoordE endPoint = null;
	// ending point of dragging
	private HTCoordS clickPoint = null;
	// clicked point

	private HTDrawNode node;
	private HTDrawNode nodeA;
	private HTDrawNode nodeH = null;
	private HTDrawNode nodeClicked;
	private HTDrawNode nodeClickedA;
	private HTDrawNode nodeClickedC;
	/**
	 *  Description of the Field
	 */
	protected volatile int fClickCount;
	/**
	 *  Description of the Field
	 */
	protected volatile String nodeID = "";
	/**
	 *  Description of the Field
	 */
	protected volatile String nodeID_A = "";

	private Applet applet;

	private static boolean isDragged = false;
	private static Frame f = null;


	/*
	 *  --- Constructor ---
	 */
	/**
	 * Constructor.
	 *
	 * @param  model    the drawing model
	 * @param  jApplet  Description of the Parameter
	 */
	HTAction(HTDraw model, Applet jApplet) {
		this.model = model;
		startPoint = new HTCoordE();
		endPoint = new HTCoordE();
		clickPoint = new HTCoordS();
		applet = jApplet;
	}


	/*
	 *  --- MouseAdapter ---
	 */
	/**
	 * Called when a user pressed the mouse button
	 * on the hyperbolic tree.
	 * Used to get the starting point of the drag.
	 *
	 * @param  e  the MouseEvent generated when clicking
	 */
	public void mousePressed(MouseEvent e) {
		if (e.isShiftDown()) {
			model.fastMode(true);
		}
		model.setHint(false);
		isDragged = true;

		startPoint.projectionStoE(e.getX(), e.getY(), model);
	}


	/**
	 * Called when a user release the mouse button
	 * on the hyperbolic tree.
	 * Used to signal the end of the translation.
	 *
	 * @param  e  not used here
	 */
	public void mouseReleased(MouseEvent e) {
		model.fastMode(false);
		model.endTranslation();
		isDragged = false;
	}


	/**
	 * Called when a user move mouse over a node on the hyperbolic tree.
	 * Used to put a hint on the corresponding node (if any) at the
	 * bellowf of the node.
	 *
	 * @param  e  the MouseEvent generated when clicking
	 */

	public void mouseEntered(MouseEvent e) {

	}


	/**
	 * Called when a user move mouse goes off a node on the hyperbolic tree.
	 * Used to drop a hint on the corresponding node (if any).
	 *
	 * @param  e  the MouseEvent generated when clicking
	 */

	public void mouseExited(MouseEvent e) {

	}


	//--- implement KeyListener
	/**
	 *  Description of the Method
	 *
	 * @param  ev  Description of the Parameter
	 */
	public void keyPressed(KeyEvent ev) {
		/*
		 *  int code = ev.getKeyCode();
		 *  switch (code) {
		 *  case KeyEvent.VK_UP:
		 *  System.out.println("Up key");
		 *  break;
		 *  case KeyEvent.VK_DOWN:
		 *  System.out.println("Down key");
		 *  break;
		 *  case KeyEvent.VK_LEFT:
		 *  System.out.println("Left key");
		 *  break;
		 *  case KeyEvent.VK_RIGHT:
		 *  System.out.println("Right key");
		 *  break;
		 *  case KeyEvent.VK_ALT:
		 *  System.out.println("1.) ALT com a tecla "+ ev.getKeyChar());
		 *  break;
		 *  case KeyEvent.VK_CONTROL:
		 *  System.out.println("CONTROL");
		 *  break;
		 *  default: System.out.println("Key press ignored");
		 *  }
		 *  if (ev.isAltDown())
		 *  System.out.println("Key Pressed "+ ev.getKeyChar());
		 */
	}


	/**
	 *  Description of the Method
	 *
	 * @param  ev  Description of the Parameter
	 */
	public void keyTyped(KeyEvent ev) {
		//System.out.println("key typed: " + ev.getKeyChar());
	}


	/**
	 *  Description of the Method
	 */
	public void edit() {

		nodeClicked.setEspecialMark(true);
		String texto = nodeClicked.editFilho();

		if (model != null) {

			String nome = "";
			String cor = "";
			String hint_A = "";
			String URL_A = "";
			String cor_t = "";

			int inx;

			inx = texto.toLowerCase().indexOf("nome:");

			if (inx >= 0) {
				for (int i = inx + 5; i < texto.length(); i++) {
					if (texto.charAt(i) == ';') {
						break;
					} else {
						nome = nome + texto.charAt(i);
					}
				}
			}
			inx = texto.toLowerCase().indexOf("cor:");
			if (inx >= 0) {
				for (int i = inx + 4; i < texto.length(); i++) {
					if (texto.charAt(i) == ';') {
						break;
					} else {
						cor = cor + texto.charAt(i);
					}
				}
			}
			inx = texto.toLowerCase().indexOf("hint:");
			if (inx >= 0) {
				for (int i = inx + 5; i < texto.length(); i++) {
					if (texto.charAt(i) == ';') {
						break;
					} else {
						hint_A = hint_A + texto.charAt(i);
					}
				}
			}

			inx = texto.toLowerCase().indexOf("url:");
			if (inx >= 0) {
				for (int i = inx + 4; i < texto.length(); i++) {
					if (texto.charAt(i) == ';') {
						break;
					} else {
						URL_A = URL_A + texto.charAt(i);
					}
				}
			}

			inx = texto.toLowerCase().indexOf("cor_texto:");
			if (inx >= 0) {
				for (int i = inx + 10; i < texto.length(); i++) {
					if (texto.charAt(i) == ';') {
						break;
					} else {
						cor_t = cor_t + texto.charAt(i);
					}
				}
			}

			model.setTextWord(nome);
			model.setCorWord(cor);
			model.setCorTextWord(cor_t);
			model.setHintWord(hint_A);
			model.setURLWord(URL_A);
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  ev  Description of the Parameter
	 */
	public void keyReleased(KeyEvent ev) {

		if (ev.getKeyCode() == KeyEvent.VK_ESCAPE) {
			if (nodeH != null) {
				model.translateToOrigin(nodeH);
				nodeH = null;
				model.executaSearchWord("");
			}
			return;
		}

		if ((model != null) && (!model.getEditedChild())) {

			if ((ev != null) && (ev.isAltDown())) {
				if ((ev.getKeyChar() == 'e') && (nodeClicked != null)) {
					edit();
					nodeClickedA = nodeClicked;
					if (model != null) {
						model.reDraw(nodeClicked.getID());
					}
				}

				if ((ev.getKeyChar() == 's') && (nodeClicked != null)) {
					if (nodeClickedA != null) {
						nodeClickedA.setEspecialMark(false);
						nodeClickedA.saveFilho("nome:" + model.getTexWord() + ";" +
								"cor:" + model.getCorWord() + ";" +
								"cor_texto:" + model.getCorTextWord() + ";" +
								"hint:" + model.getHintWord() + ";" +
								"url:" + model.getURLWord());
						if (model != null) {
							model.reDraw(node.getID());
						}
					}
				}

				if ((ev.getKeyChar() == 'i') && (nodeClicked != null)) {
					nodeClicked.setEspecialMark(false);
					nodeClicked.insereFilho();
					if (model != null) {
						if (node != null) {
							model.reDraw(node.getID());
						} else {
							model.reDraw();
						}
					}
				}

				if ((ev.getKeyChar() == 'd') && (nodeClicked != null)) {

					if (f == null) {
						f = new Frame("Ateno");
						f.setSize(100, 100);
						f.show();
					}
					f.setVisible(false);

					// Create an instance of InfoDialog, with title and message specified
					HTDialogYesNo d =
							new HTDialogYesNo(f, "Informao a respeito da remoo",
							"Voc tem certeza que deseja remover este n?",
							"Sim",
							"No",
							"Cancelar");

					// Register an action listener for the dialog.  This one just prints
					// the results out to the console.
					d.addActionListener(
						new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								String answer = e.getActionCommand();

								if (answer.equals("yes")) {
									nodeClicked.deleteFilho();
									nodeClicked = null;
									if ((model != null)) {
										model.reDraw();
									}
								}
								f.setVisible(false);
							}
						});

					// Now pop the dialog up.  It will pop itself down automatically.
					d.show();

				}

				if ((ev.getKeyChar() == 'c') && (nodeClicked != null)) {
					nodeClicked.setEspecialMark(true);
					nodeClickedC = nodeClicked;
					nodeClickedA = nodeClicked;
					model.setRamo(nodeClicked.getID());
					System.out.println("COPIANDO ...." + nodeClicked.getID());
					nodeClicked = null;
				}
				if ((ev.getKeyChar() == 'v') && (nodeClicked != null) && (nodeClicked != nodeClickedC)) {
					nodeClicked.setEspecialMark(false);
					if (model != null) {
						if ((nodeClickedC != null) && (nodeClickedC.findRamo(nodeClicked.getID()) == null)) {
							nodeClickedC.setEspecialMark(false);
							nodeClicked.setEspecialMark(false);
							if (model.getRamo() != null) {
								nodeClicked.insereRamo(model.getRamo());
							}
						} else {
							System.out.println("Cpia no permitida");
						}

						nodeClickedC = null;
						nodeClickedA = null;

						System.out.println("COLANDO EM ...." + nodeClicked.getID() + " O RAMO " + model.getRamo());
						model.setRamo(null);
						nodeClicked.setInseredChild(true);
						if (nodeClicked != null) {
							model.reDraw(nodeClicked.getID());
						} else {
							model.reDraw();
						}
					}
					nodeClicked = null;
				}

				// Inserindo Espelho **************************************************************************

				if ((ev.getKeyChar() == 'm') && (nodeClicked != null) && (nodeClicked != nodeClickedC)) {
					nodeClicked.setEspecialMark(false);
					if (model != null) {
						if ((nodeClickedC != null) && (nodeClickedC.findRamo(nodeClicked.getID()) == null)) {
							nodeClickedC.setEspecialMark(false);
							nodeClicked.setEspecialMark(false);
							if (model.getRamo() != null) {
								nodeClicked.insereEspelho(model.getRamo());
							}
						} else {
							System.out.println("Cpia no permitida");
						}

						nodeClickedC = null;
						nodeClickedA = null;

						System.out.println("COLANDO ESPELHO EM ...." + nodeClicked.getID() + " O RAMO " + model.getRamo());
						model.setRamo(null);
						nodeClicked.setInseredChild(true);
						if (nodeClicked != null) {
							model.reDraw(nodeClicked.getID());
						} else {
							model.reDraw();
						}

					}
					nodeClicked = null;
				}

				// Inserindo Link Escondido **************************************************************************

				if ((ev.getKeyChar() == 'h') && (nodeClicked != null) && (nodeClicked != nodeClickedC)) {
					nodeClicked.setEspecialMark(false);
					if (model != null) {
						if ((nodeClickedC != null) && (nodeClickedC.findRamo(nodeClicked.getID()) == null)) {
							nodeClickedC.setEspecialMark(false);
							nodeClicked.setEspecialMark(false);
							if (model.getRamo() != null) {
								nodeClicked.insereHideLink(model.getRamo());
							}
						} else {
							System.out.println("Cpia no permitida");
						}

						nodeClickedC = null;
						nodeClickedA = null;

						System.out.println("COLANDO LINK ESCONDIDO EM ...." + nodeClicked.getID() + " O RAMO " + model.getRamo());
						model.setRamo(null);
						nodeClicked.setInseredChild(true);
						if (nodeClicked != null) {
							model.reDraw(nodeClicked.getID());
						} else {
							model.reDraw();
						}

					}
					nodeClicked = null;
				}

			}
			if ((ev != null) && (ev.isControlDown())) {
				if ((ev.getKeyChar() == 'c') && (nodeClicked != null)) {
					nodeClicked.setEspecialMark(true);
					model.setRamo(nodeClicked.getID());
					System.out.println("COPIANDO ...." + nodeClicked.getID());
				}


				if ((ev.getKeyChar() == 'v') && (nodeClicked != null)) {
					nodeClicked.setEspecialMark(true);
					nodeClicked.insereRamo(model.getRamo());
					System.out.println("COLANDO EM ...." + nodeClicked.getID() + " O RAMO " + model.getRamo());
					//model.setRamo(null);
					nodeClicked.setInseredChild(true);
					if (model != null) {
						model.reDraw();
					}
				}
				// Inserindo Espelho **************************************************************************
				if ((ev.getKeyChar() == 'm') && (nodeClicked != null)) {
					nodeClicked.setEspecialMark(true);
					nodeClicked.insereEspelho(model.getRamo());
					nodeClicked.setInseredChild(true);
					System.out.println("COLANDO ESPELHO EM ...." + nodeClicked.getID() + " O RAMO " + model.getRamo());
					//model.setRamo(null);
					if (model != null) {
						model.reDraw();
					}
				}
				// Inserindo Link Escondido **************************************************************************
				if ((ev.getKeyChar() == 'h') && (nodeClicked != null)) {
					nodeClicked.setEspecialMark(true);
					nodeClicked.insereHideLink(model.getRamo());
					nodeClicked.setInseredChild(true);
					//System.out.println("COLANDO LINK ESCONDIDO EM ...." + nodeClicked.getID() + " O RAMO " + model.getRamo());
					//model.setRamo(null);
					if (model != null) {
						model.reDraw();
					}
				}

			}
		}
	}


	/**
	 * Called when a user clicked on the hyperbolic tree.
	 * Used to put the corresponding node (if any) at the
	 * center of the hyperbolic tree.
	 *
	 * @param  e  the MouseEvent generated when clicking
	 */
	public void mouseClicked(MouseEvent e) {

		model.setHint(false);

		startPoint.projectionStoE(e.getX(), e.getY(), model);
		// Silvio

		if (e.isShiftDown()) {
			model.restore();
		} else {

			clickPoint.x = e.getX();
			clickPoint.y = e.getY();

			node = model.findNode(clickPoint);

			// 13/09/2004

			int res1 = 0;

			if (node != null) {
				res1 = node.verifyFolder(clickPoint);
			}

			if (res1 == 1) {
				String hasFolder = node.getNodeClientData();

				if (!node.isLeafNode()) {
					if (hasFolder.equals("11")) {
						node.setNodeClientData("10");
					}

					if (hasFolder.equals("10")) {
						node.setNodeClientData("11");
					}
				}
			}

			// END

			if (node != null) {
				nodeClicked = node;
				node.setEspecialMark(true);
			} else {
				nodeClicked = null;
			}

			if (node == null) {
				int res = model.verifyHomeClick(clickPoint);

				if (res == 1) {
					// Asking for Help
					String help = model.getHelpFile();

					if (help.length() > 2) {
						boolean UrlOnError = false;
						URL userUrl = null;
						try {
							userUrl = new URL(help);
						} catch (Exception urlError) {
							UrlOnError = true;
						}
						if (!UrlOnError) {
							applet.getAppletContext().showDocument(userUrl, "_blank");
						}
					}
				}

				if ((clickPoint != null) && (model.verifyOKClick(clickPoint))) {
					if (nodeClickedA != null) {
						if (model != null) {
							model.setEditedChild(false);
							model.reDraw();
							nodeClickedA.setEspecialMark(false);
							nodeClickedA.saveFilho("nome:" + model.getTexWord() + ";" +
									"cor:" + model.getCorWord() + ";" +
									"cor_texto:" + model.getCorTextWord() + ";" +
									"hint:" + model.getHintWord() + ";" +
									"url:" + model.getURLWord());

							nodeClickedA = null;

						}
					}
				}
				if ((clickPoint != null) && (model.verifyCANCELClick(clickPoint))) {
					if (nodeClickedA != null) {
						if (model != null) {
							model.setEditedChild(false);
							model.reDraw();
							nodeClickedA.setEspecialMark(false);
							nodeClickedA = null;
						}
					}
				}
				if ((clickPoint != null) && (model.verifyAplicarClick(clickPoint))) {
					if (nodeClickedA != null) {
						if (model != null) {
							model.setEditedChild(false);
							model.reDraw();
							nodeClickedA.setEspecialMark(false);
							nodeClickedA.aplicarCoresFilhos("cor:" + model.getCorWord() + ";" +
									"cor_texto:" + model.getCorTextWord());
							nodeClickedA = null;
						}
					}
				}
			}

			if ((fClickCount = e.getClickCount()) == 1) {

				new Thread(
					new Runnable() {
						public void run() {
							try {
								Thread.sleep(600);
								if (fClickCount == 1) {
									fClickCount = 0;
									if (node != null) {

										// Silvio em 28/09/2004
										// Vou verificar se o n  do tipo link escondido

										int posH = node.getID().indexOf("_H_");
										if (posH >= 0) {
											HTDrawNode nodeC;
											String idNode = node.getID().substring(0, posH);
											nodeC = model.findNode(idNode);
											if (nodeC != null) {

												model.executaSearchWord(node.getID());//idNode);
												model.translateToOrigin(nodeC);
												nodeH = node;
											}
										} else {

											posH = -1;

											if (nodeH != null) {
												posH = nodeH.getID().indexOf("_H_");
											}

											if (posH >= 0) {
												String idNode = nodeH.getID().substring(0, posH);
												if (node.getID().compareTo(idNode) == 0) {
													model.translateToOrigin(nodeH);
												} else {
													model.translateToOrigin(node);
												}

												nodeH = null;
												model.executaSearchWord("");

											} else {
												model.translateToOrigin(node);
											}
										}

										// **** SILVIO

										if (nodeClickedA != null) {
											if ((model != null) && (model.getEditedChild())) {
												//model.setEditedChild(false);
												model.reDraw();
												nodeClickedA.setEspecialMark(false);
												nodeClickedA.saveFilho("nome:" + model.getTexWord() + ";" +
														"cor:" + model.getCorWord() + ";" +
														"cor_texto:" + model.getCorTextWord() + ";" +
														"hint:" + model.getHintWord() + ";" +
														"url:" + model.getURLWord());

												nodeClickedA = null;

												nodeClicked = node;
												edit();
												nodeClickedA = nodeClicked;
												if (model != null) {
													model.reDraw(nodeClicked.getID());
												}
											}
										}
									}
								} else {
									if ((node != null) && (model != null) && (!model.getEditedChild())) {

										node = model.findNode(clickPoint);
										node.setEspecialMark(false);
										node.insereFilho();
										model.reDraw(node.getID());

										/*
										 *  boolean UrlOnError = false;
										 *  URL userUrl  = null;
										 *  String adr;
										 *  adr = node.getURL();
										 *  String base = model.getUrlAdr();
										 *  if (base.length() > 2) {
										 *  adr = base + "/" + adr;
										 *  }
										 *  try
										 *  {
										 *  userUrl = new URL(adr);
										 *  }
										 *  catch (Exception urlError)
										 *  {
										 *  UrlOnError = true;
										 *  }
										 *  if (!UrlOnError) {
										 *  applet.getAppletContext().showDocument(userUrl,"_blank");
										 *  }
										 *  else {
										 *  System.out.println("URL Invlida " +  adr);
										 *  }
										 */
										fClickCount = 0;
									}
								}
							} catch (Exception ex) {} finally {}
						}
					}).start();
			}
		}

	}


	/*
	 *  --- MouseMotionListener ---
	 */
	/**
	 * Called when a used drag the mouse on the hyperbolic tree.
	 * Used to translate the hypertree, thus moving the focus.
	 *
	 * @param  e  the MouseEvent generated when draging
	 */
	public void mouseDragged(MouseEvent e) {
		model.setHint(false);
		model.fastMode(true);
		isDragged = true;

		if (startPoint.isValid()) {
			endPoint.projectionStoE(e.getX(), e.getY(), model);
			if (endPoint.isValid()) {
				model.translate(startPoint, endPoint);
			}
		}
	}


	/**
	 * Called when the mouse mouve into the hyperbolic tree.
	 * Not used here.
	 *
	 * @param  e  the MouseEvent generated when mouving
	 */
	public void mouseMoved(MouseEvent e) {

		Point pt = e.getPoint();

		clickPoint.x = pt.x;
		clickPoint.y = pt.y;

		if ((fClickCount > 0) || (isDragged)) {
			return;
		}

		if (node == null) {
			nodeA = null;
		} else {
			if (nodeID.equals(nodeID_A) == false) {
				if (nodeA != null) {
					nodeA.setHasHint(false);
					model.setHint(false);
					model.reDraw();
				}
				nodeA = node;
				nodeID_A = nodeID;
			}
		}

		node = model.findNode(clickPoint);

		if (node != null) {
			if (nodeA != null) {
				nodeA.setEspecialMark(false);
			}

			//node.setEspecialMark(true);

			if ((nodeID.equals(nodeID_A)) || (nodeID_A.equals(""))) {
				nodeID_A = nodeID;
			} else {
				nodeID_A = "";

			}
			nodeID = node.getID();

		} else {
			if (nodeA != null) {
				nodeA.setEspecialMark(false);
				nodeA.setHasHint(false);
				//nodeA.setHintPai(false);
				model.setHint(false);
				model.reDraw();
			}

			nodeID_A = nodeID;
			nodeID = "";

		}

		if (node != null) {
			new Thread(
				new Runnable() {
					public void run() {
						try {
							if ((nodeID_A.equals("")) || (nodeID.equals(nodeID_A))) {
								Thread.sleep(600);
								if ((fClickCount > 0) || (isDragged)) {
									node.setHasHint(false);
									//node.setHintPai(false);
									model.setHint(false);
									return;
								}
								if ((nodeID.equals("") == false) && (nodeID.equals(nodeID_A))) {
									node.setHasHint(true);
									//node.setHintPai(true);
									model.setHint(true);
								}
							}
							model.reDraw();
						} catch (Exception ex) {} finally {}
					}
				}).start();
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  evt  Description of the Parameter
	 */
	public void actionPerformed(ActionEvent evt) {

		if (model.getGravar() == evt.getSource()) {
			if (model != null) {
				model.grava();
			}
		}

		if (model.getRecuperar() == evt.getSource()) {
			if (model != null) {
				if (model.getHasUndo()) {
					model.undoRecupera();
					model.reDraw();
				}
			}
		}

		if (model.getColarEscondido()== evt.getSource()) {
				// Inserindo Link Escondido **************************************************************************
				if ((model != null) && (nodeClicked != null)) {
					nodeClicked.setEspecialMark(true);
					nodeClicked.insereHideLink(model.getRamo());
					nodeClicked.setInseredChild(true);
					if (model != null) {
						model.reDraw();
					}
				}

		}

		if (model.getInserir() == evt.getSource()) {
			if ((nodeClicked != null)) {
				nodeClicked.setEspecialMark(false);
				nodeClicked.insereFilho();
				if (model != null) {
					model.reDraw(nodeClicked.getID());
				}
				//if (node != null)
				//	model.reDraw(node.getID());
				//else
				//	model.reDraw();
			}
		}

		if (model.getEditar() == evt.getSource()) {
			if ((nodeClicked != null)) {
				edit();
				nodeClickedA = nodeClicked;
				if (model != null) {
					model.reDraw(nodeClicked.getID());
				}
			}
		}

		if (model.getCopiar() == evt.getSource()) {
			if ((nodeClicked != null)) {
				nodeClicked.setEspecialMark(true);
				nodeClickedC = nodeClicked;
				nodeClickedA = nodeClicked;
				model.setRamo(nodeClicked.getID());
				System.out.println("COPIANDO ...." + nodeClicked.getID());
				nodeClicked = null;
			}
		}

		if (model.getColar() == evt.getSource()) {
			if ((nodeClicked != null) && (nodeClicked != nodeClickedC)) {
				nodeClicked.setEspecialMark(false);
				if (model != null) {
					if ((nodeClickedC != null) && (nodeClickedC.findRamo(nodeClicked.getID()) == null)) {
						nodeClickedC.setEspecialMark(false);
						nodeClicked.setEspecialMark(false);
						if (model.getRamo() != null) {
							model.undoGrava();
							nodeClicked.insereRamo(model.getRamo());
						}
					} else {
						System.out.println("Cpia no permitida");
					}

					nodeClickedC = null;
					nodeClickedA = null;

					System.out.println("COLANDO EM ...." + nodeClicked.getID() + " O RAMO " + model.getRamo());
					model.setRamo(null);
					if (nodeClicked != null) {
						model.reDraw(nodeClicked.getID());
					} else {
						model.reDraw();
					}
				}
				nodeClicked = null;
			}
		}

		if (model.getCentralizar() == evt.getSource()) {
			model.goOrigem();
		}

		if (model.getDeletar() == evt.getSource()) {
			if ((nodeClicked != null)) {
				if (f == null) {
					f = new Frame("Ateno");
					f.setSize(100, 100);
					f.show();
				}
				f.setVisible(false);

				// Create an instance of InfoDialog, with title and message specified
				HTDialogYesNo d =
						new HTDialogYesNo(f, "Informao a respeito da remoo",
						"Voc tem certeza que deseja remover o n " + nodeClicked.getName() + "(" + nodeClicked.getID() + ")?",
						"Sim",
						"No",
						"Cancelar");

				// Register an action listener for the dialog.  This one just prints
				// the results out to the console.
				d.addActionListener(
					new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							String answer = e.getActionCommand();

							if (answer.equals("yes")) {

								model.undoGrava();

								nodeClicked.deleteFilho();

								nodeClicked = null;
								if ((model != null)) {
									model.reDraw();
								}
							}
							f.setVisible(false);
						}
					});

				// Now pop the dialog up.  It will pop itself down automatically.
				d.show();

			}
		}

		if (model.getAplicarT() == evt.getSource()) {
			if (nodeClickedA != null) {
				if (model != null) {
					model.undoGrava();

					//model.setEditedChild(false);
					//nodeClickedA.setEspecialMark(false);
					nodeClickedA.aplicarCoresFilhos("cor_texto:" + model.getCorTextWord());
					//nodeClickedA = null;
					model.reDraw();
				}
			}
		}
		if (model.getAplicarF() == evt.getSource()) {
			if (nodeClickedA != null) {
				if (model != null) {
					model.undoGrava();
					//model.setEditedChild(false);
					//nodeClickedA.setEspecialMark(false);
					nodeClickedA.aplicarCoresFilhos("cor:" + model.getCorWord());
					//nodeClickedA = null;
					model.reDraw();

				}
			}
		}

		if (model.getCorT() == evt.getSource()) {
			HTColorPicker colorPicker = new HTColorPicker(model.getCorTextF());
		}
		if (model.getCorF() == evt.getSource()) {
			HTColorPicker colorPicker = new HTColorPicker(model.getCorFieldF());
		}
		if (model.getOk() == evt.getSource()) {
			if (nodeClickedA != null) {
				if (model != null) {
					model.setEditedChild(false);
					model.reDraw();
					nodeClickedA.setEspecialMark(false);
					nodeClickedA.saveFilho("nome:" + model.getTexWord() + ";" +
							"cor:" + model.getCorWord() + ";" +
							"cor_texto:" + model.getCorTextWord() + ";" +
							"hint:" + model.getHintWord() + ";" +
							"url:" + model.getURLWord());

					nodeClickedA = null;

				}
			}
		}
		if (model.getCancel() == evt.getSource()) {
			if (nodeClickedA != null) {
				if (model != null) {
					model.setEditedChild(false);
					model.reDraw();
					nodeClickedA.setEspecialMark(false);
					nodeClickedA = null;
				}
			}
		}
	}

}

