/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 * This code was modified by Silvio Evangelista e Kleber Sampaio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

//package hypertree;

import java.util.Enumeration;


/**
 * The HTNode interface should be implemented by 
 * object that are node of the tree that want to be 
 * displayed in the TreeMap.
 * <P>
 * If you have already a tree structure, just implements
 * this interface in node of the tree.
 */
public interface HTNode {

    /**
     * Returns the children of this node
     * in an Enumeration.
     * If this object does not have children,
     * it should return an empty Enumeration,
     * not <CODE>null</CODE>.
     * All objects contained in the Enumeration
     * should implements HTNode.
     *
     * @return    an Enumeration containing childs of this node
     */
    public Enumeration children();
    public Enumeration childrenNode();


    /**
     * Checks if this node is a leaf or not.
     * A node could have no children and still not
     * be a leaf.
     *
     * @return    <CODE>true</CODE> if this node is a leaf;
     *            <CODE>false</CODE> otherwise
     */
    public boolean isLeaf();

    public boolean isLeafNode();

    
    /**
     * Returns the name of this node.
     * Used to display a label in the hyperbolic tree.
     *
     * @return    the name of this node
     */
    public String getName();

    /**
     * Returns the color of this node.
     * Used to display a color in the hyperbolic tree.
     *
     * @return    the color of this node
     */
    public String getColor();

    /**
     * Returns the hint of this node.
     * Used to display a hint in the hyperbolic tree.
     *
     * @return    the hint of this node
     */
    public String getHint();

    /**
     * Returns the text color of this node.
     * Used to display a text color in the hyperbolic tree.
     *
     * @return    the text color of this node
     */
    public String getTextColor();

    /**
     * Returns the URL of this node.
     * Used to display a URL in the hyperbolic tree.
     *
     * @return    the URL  of this node
     */
    public String getURL();

    /**
     * Returns the ID of this node.
     *    
     * @return    the ID of this node
     */
    public String getID();

    public void setLeafNode(boolean v);


    /**
     * set true or false Highlight of the node.
     *
     * 
     */

    void setEspecialMark(boolean t);
   

    /**
     * get Highlight of the node.
     *
     * 
     */

    boolean getEspecialMark();

    /**
     * Returns the attributes hasHint of the node.
     * @return boolean 
     */
    public boolean getHasHint();
    
   /**
     * Returns the attributes hasData of the node.
     * @return boolean 
     */

    public boolean getHasData();

    /**
     * Set the attributes hint of the node.
     * @void
     */
    public void setHasHint(boolean v);

    /**
     * Set the attributes hasData of the node.
     * @void
     */
    public void setHasData(boolean v);

   /**
     * Set the attributes hasArc of the node.
     * @void
     */
    public void setHasArc(boolean v);

    /**
     * get the attributes hasArc of the node.
     * @void
     */
    public boolean getHasArc();


    /**
     * Insert a new childen on actual node
     * @void
     */

    public void insereFilho(int nchilds);

    public void deleteFilho(String id);

    public void insereRamo(HTNode ramo, int nNodes);

    public HTNode findPai(String id);
    
    //public HTNode findPai(HTNode filho);

    public HTNode findRamo(String id);

    public String editFilho();

    public void saveFilho(String texto);

    public void aplicarCoresFilhos(String cores);

    // 13/09/2004

    public String getNodeClientData();

    public void setNodeClientData(String v);

    // END


}

