/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 * This code was modified by Silvio Evangelista e Kleber Sampaio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
//package hypertree;

import java.awt.*;
import java.util.*;

/**
 * The HTNodeLabel class implements the drawed label
 * representing a node.
 *
 * @author     silvio
 * @created    29 de Setembro de 2004
 */
class HTNodeLabel {

	private HTDrawNode node = null;
	// represented node
	private int x = 0;
	// x up-left corner
	private int y = 0;
	// y up-left corner
	private int width = 0;
	// width of the label
	private int height = 0;
	// height of the label
	private boolean active = false;
	// should be drawed ?


	/*
	 *  ---  Constructor ---
	 */
	/**
	 * Constructor.
	 *
	 *
	 * @param  node  the represented node
	 */
	HTNodeLabel(HTDrawNode node) {
		this.node = node;
	}


	/*
	 *  --- Draw ---
	 */
	/**
	 * Draw this label, if there is enought space.
	 *
	 * @param  g         the graphic context
	 * @param  fastMode  Description of the Parameter
	 */
	void draw(Graphics g, boolean fastMode) {

		int fh = node.getCharHeight();
		int space = node.getSpace();

		if (space >= fh) {
			active = true;
			HTCoordS zs = node.getScreenCoordinates();
			String name = node.getName();
			String corNo = node.getColor();
			boolean hasData = node.getHasData();
			boolean border = node.getEspecialMark();
			String corTexto = node.getTextColor();

			char[] nameC = name.toCharArray();
			int nameLength = nameC.length;
			int nameWidth = node.getStringWidth(nameC, 0, nameLength);

			//int nameWidth = node.getCharWidth()*nameLength;

			while ((nameWidth >= space) && (nameLength > 0)) {
				nameLength--;
				//nameWidth = node.getCharWidth()*nameLength;
				nameWidth = node.getStringWidth(nameC, 0, nameLength);
			}

			fh = fh + 4;
			// Silvio: Aumentando a altura do n. Este comando no existia.

			int typeNode = 0;
			// Box

			if (node != null && node.getID() != null && node.getID().indexOf("_H_") >= 0) {
				typeNode = 1;
			}
			//Elipse

			height = fh;
			width = nameWidth + 5;

			if (typeNode == 1) {
				width = width + 4;
				height = height + 4;

			}
			x = zs.x - (width / 2);
			y = zs.y - (fh / 2);

			if ((corNo != null) && (corNo.length() > 6)) {
				g.setColor(new Color(Color.decode(corNo).getRGB()));
			} else {
				g.setColor(new Color(Color.decode("0xFF0000").getRGB()));
			}

			if (node.getBorderType().toLowerCase().equals("normal") || fastMode) {
				if (typeNode == 0) {
					g.fillRect(x, y, width + 5, height);
				}
				if (typeNode == 1) {
					g.fillOval(x, y, width, height);
				}
			} else {
				if (typeNode == 0) {
					g.fillRoundRect(x, y, width + 5, height, 8, 8);
				}
				if (typeNode == 1) {
					g.fillOval(x, y, width, height);
				}
			}

			if (border) {
				g.setColor(Color.orange);
				Rectangle r = new Rectangle(x + 2, y + 2, width - 4, height - 4);
				if (node.getBorderType().toLowerCase().equals("normal") || fastMode) {
					if (typeNode == 0) {
						g.fillRect(r.x, r.y, r.width + 5, r.height);
					}
					if (typeNode == 1) {
						g.fillOval(r.x, r.y, r.width, r.height);
					}
				} else {
					if (typeNode == 0) {
						g.fillRoundRect(r.x, r.y, r.width + 5, r.height, 8, 8);
					}
					if (typeNode == 1) {
						g.fillOval(r.x, r.y, r.width, r.height);
					}
				}

			} else {
				g.setColor(Color.black);

				if (node.getBorderType().toLowerCase().equals("normal") || fastMode) {
					if (typeNode == 0) {
						g.drawRect(x, y, width + 5, height);
					}
					if (typeNode == 1) {
						g.drawOval(x, y, width, height);
					}
				} else {
					if (typeNode == 0) {
						g.drawRoundRect(x, y, width + 5, height, 8, 8);
					}
					if (typeNode == 1) {
						g.drawOval(x, y, width, height);
					}
				}
			}

			// 13/09/2004
			String hasFolder = node.getNodeClientData();

			if (!node.isLeafNode()) {
				if (hasFolder.equals("11")) {
					g.setColor(new Color(Color.decode(corTexto).getRGB()));
					//g.fillOval(x+width-2, y+2, 5, 5);
					if (typeNode == 0) {
						g.fillRect(x + width - 2, y + 2, 5, 5);
					}
				}

				if (hasFolder.equals("10")) {
					g.setColor(new Color(Color.decode(corTexto).getRGB()));
					if (typeNode == 0) {
						g.drawRect(x + width - 2, y + 2, 5, 5);
					}
					//g.drawOval(x+width-2, y+2, 5, 5);

				}
			}
			// END


			//drawRect(x, y, width, height);


			if (hasData) {
				//g.setColor(Color.red);
				g.setColor(new Color(Color.decode(node.getSearchColor()).getRGB()));
				g.fillOval(x + width / 2 - 3, y + fh / 2 + 3, 7, 7);
			}

			int sx = zs.x - (nameWidth / 2);
			int sd = node.getAscent() + (node.getLeading() / 2);
			int sy = y + sd;
			// model.getAscent() + (model.getLeading() / 2);

			if ((corTexto != null) && (corTexto.length() > 6)) {
				g.setColor(new Color(Color.decode(corTexto).getRGB()));
			} else {
				g.setColor(new Color(Color.decode("0x000000").getRGB()));
			}

			if (hasData) {
				g.drawString(new String(nameC, 0, nameLength), sx, sy - 1);
			} else {
				g.drawString(new String(nameC, 0, nameLength), sx, sy + 2);
			}

			g.setColor(Color.black);

		} else {
			active = false;
		}
	}


	/**
	 * Draw this hint of the node, if asked.
	 *
	 * @param  g  the graphic context
	 */

	void drawHint(Graphics g) {

		int fh = node.getCharHeight();
		int space = node.getSpace();
		if (space >= fh) {

			active = true;

			boolean hasHint = node.getHasHint();

			//HTModelNode model  = (HTModelNode) node.getHTModelNode(); // get Model

			boolean hasHintPai = node.getHasHintPai();

			//if (model != null) {
			//	HTNode pai        		= (HTNode) model.getParentChild();
			//	if (pai != null)  hasHintPai  = pai.getHasHint();
			//}


			int panelW = node.getWidthPanel();
			int panelH = node.getHeighthPanel();

			if (hasHint) {

				HTCoordS zs = node.getScreenCoordinates();
				String name = node.getName();
				String URL = node.getURL();

				char[] nameC = name.toCharArray();
				int nameLength = nameC.length;

				int nameWidth = node.getStringWidth(nameC, 0, nameLength);
				//int nameWidth  = node.getCharWidth()*nameLength

				while ((nameWidth >= space) && (nameLength > 0)) {
					nameLength--;
					nameWidth = node.getStringWidth(nameC, 0, nameLength);

				}

				height = fh;
				width = nameWidth + 10;

				x = zs.x - (width / 2);
				y = zs.y - (fh / 2);

				int sx = zs.x - (nameWidth / 2);
				int sd = node.getAscent() + (node.getLeading() / 2);
				int sy = y + sd;
				int delX;
				int delY;

				// Vou processar as Linhas

				String hint = node.getHint();

				String hintA = "";

				if (hint.toLowerCase().indexOf("&crlf") < 0) {
					int pos = 0;
					for (int i = 0; i < hint.length(); i++) {
						pos++;

						hintA = hintA + hint.charAt(i);

						if (pos > 60 && hint.charAt(i) == ' ') {
							hintA = hintA + "&crlf";
							pos = 0;
						}
					}
					hint = hintA;

				}

				String texto = name + "&crlf " + hint;
				//node.getHint();// + "&crlf " + "&crlf "+ URL;

				HTMessage s = new HTMessage(texto);
				int tamanho = node.getStringWidth(s.getLine(s.getIdM())) + 4;
				int totalHeight = s.getNumLines() * height;

				int meioX = 0;

				int fromTop = sy + sd + 8;

				if (nameWidth > tamanho) {
					tamanho = nameWidth + 8;
				}

				g.setColor(new Color(Color.decode(node.getHintColor()).getRGB()));

				if ((tamanho + 16 + x - meioX) < panelW) {
					delX = meioX;
				} else {
					delX = meioX + ((tamanho + 16 + x - meioX) - panelW);
				}

				if ((y + sd + 16 + totalHeight + 2) < panelH) {
					delY = 0;
				} else {
					delY = y + sd + 16 + totalHeight + 2 - panelH;
				}

				g.fillRect(x - delX, y + sd + 8 - delY, tamanho + 16, (totalHeight + 2));
				g.setColor(Color.black);
				g.drawRect(x - delX, y + sd + 8 - delY, tamanho + 16, (totalHeight + 2));

				for (int index = 0; index < s.getNumLines(); index++) {
					if (index == 0) {
						g.setColor(Color.black);
						g.drawRect(x - delX, y + sd + 8 - delY, tamanho + 16, height);

						if ((node.getColor() != null) && (node.getColor().length() > 6)) {
							g.setColor(new Color(Color.decode(node.getColor()).getRGB()));
						} else {
							g.setColor(new Color(Color.decode("0xFF0000").getRGB()));
						}

						g.fillRect(x - delX, y + sd + 8 - delY, tamanho + 16, height);


						if ((node.getTextColor() != null) && (node.getTextColor().length() > 6)) {
							g.setColor(new Color(Color.decode(node.getTextColor()).getRGB()));
						} else {
							g.setColor(new Color(Color.decode("0x000000").getRGB()));
						}
					}

					if (node.getHasData()) {
						drawMarkedHint(s.getLine(index), sx, delX, fromTop, delY, g, index);
					} else {

						if (index == 0) {
							g.drawString(s.getLine(index), sx - delX, fromTop - delY);
						} else {
							g.setColor(Color.black);
							g.drawString(s.getLine(index), sx - delX, fromTop - delY);

						}
					}
					fromTop += height;
				}

				g.setColor(Color.black);
			}
		} else {
			active = false;
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  linha    Description of the Parameter
	 * @param  sx       Description of the Parameter
	 * @param  delX     Description of the Parameter
	 * @param  fromTop  Description of the Parameter
	 * @param  delY     Description of the Parameter
	 * @param  g        Description of the Parameter
	 * @param  line     Description of the Parameter
	 */
	void drawMarkedHint(String linha, int sx, int delX, int fromTop, int delY, Graphics g, int line) {
		StringTokenizer tokenizer = new StringTokenizer(linha, " ");
		int numWords = tokenizer.countTokens();
		String word;
		String word1;
		String wordBusca = node.getSearchWord().toLowerCase();
		int pos = 0;
		int nAnds = node.getNumAnds();
		// 09/09/2004

		for (int index = 0; index < numWords; index++) {

			word = ((String) tokenizer.nextToken());
			word1 = word + " ";

			char[] nameC = word1.toCharArray();
			int nameLength = nameC.length;
			int nameWidth = node.getStringWidth(nameC, 0, nameLength);
			//int nameWidth  = node.getCharWidth()*nameLength;

			// 09/09/2004
			boolean hasW = false;
			for (int i = 0; i < nAnds; i++) {
				wordBusca = node.getWordI(i);
				if ((wordBusca != null) && (word.trim().toLowerCase().startsWith(wordBusca.trim()))) {
					hasW = true;
					break;
				}
			}

			if (hasW) {
				g.setColor(Color.red);
				g.drawString(word1, sx + pos - delX, fromTop - delY);
			} else {
				g.setColor(Color.black);
				g.drawString(word1, sx + pos - delX, fromTop - delY);
			}

			pos += nameWidth;
		}
	}



	/*
	 *  --- Zone containing ---
	 */
	/**
	 * Is the given HTCoordS within this label ?
	 *
	 * @param  zs  Description of the Parameter
	 * @return     <CODE>true</CODE> if it is,
	 *            <CODE>false</CODE> otherwise
	 */
	boolean contains(HTCoordS zs) {
		if (active) {
			if ((zs.x >= x) && (zs.x <= (x + width + 5)) &&
					(zs.y >= y) && (zs.y <= (y + height))) {
				return true;
			} else {
				return false;
			}
		} else {
			return node.getScreenCoordinates().contains(zs);
		}
	}


	// 13/09/2004

	/**
	 *  Description of the Method
	 *
	 * @param  zs  Description of the Parameter
	 * @return     Description of the Return Value
	 */
	int verifyFolder(HTCoordS zs) {

		if (node != null) {
			int fh = node.getCharHeight();
			int space = node.getSpace();

			String name = node.getName();
			char[] nameC = name.toCharArray();
			int nameLength = nameC.length;
			int nameWidth = node.getStringWidth(nameC, 0, nameLength);
			while ((nameWidth >= space) && (nameLength > 0)) {
				nameLength--;
				nameWidth = node.getStringWidth(nameC, 0, nameLength);
			}

			width = nameWidth + 5;

			if (active) {
				if ((zs.x >= x + width - 7) && (zs.x <= (x + width + 7)) &&
						(zs.y >= y - 4) && (zs.y <= (y + 8))) {
					return 1;
				} else {
					return 0;
				}
			} else {
				return 0;
			}

		} else {
			return 0;
		}
	}


	// END


	/*
	 *  --- ToString ---
	 */
	/**
	 * Returns a string representation of the object.
	 *
	 * @return    a String representation of the object
	 */
	public String toString() {
		String result = "label of " + node.getName() +
				"\n\tx = " + x + " : y = " + y +
				"\n\tw = " + width + " : h = " + height;
		return result;
	}

}
