/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 * This code was modified by Silvio Evangelista e Kleber Sampaio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

//package hypertree;

import java.awt.*;

import javax.swing.*;
import java.applet.*;
import java.util.*;



/**
 * The HTView class implements a view of the HyperTree.
 */
public class HTView
    extends  HTDoubleBuffer { // JPanel or Panel

    private String     actualNode = null;
    private HTModel    model  = null; // the tree model represented
    private HTDraw     draw   = null; // the drawing model
    private HTAction   action = null; // action manager
    private boolean    hasHint= false;
    private TextField  textField;
    private TextField  corField;
    private TextField  corText;
    private TextField  hintField;
    private TextField  urlField;
    private TextField  searchField;
    private String     vString[] = new String[16];
    private int nAnds = 0;

    private Label  textLabel;
    private Label  corLabel;
    private Label  corTextLabel;
    private Label  hintLabel;
    private Label  urlLabel;
    private Panel  panel;

    private Button corT;
    private Button corF;
    private Button aplicarT;
    private Button aplicarF;
    private Button ok;
    private Button cancel;

    private JButton deletar;
    private JButton inserir;
    private JButton editar;
    private JButton copiar;
    private JButton colar;
    private JButton centralizar;
    private JButton recuperar;
    private JButton gravar;
    private JButton gravarXML;
    private JButton colarEscondido;


    private boolean    isSearch    = false;
    private String     hintColor   = "0xF0FFF0";
    private String     centerNode  = null;
    private String     backColor   = "0xDDDDDD";
    private String     backColorEdit   = "0xDDDDDD";
    private String     linkColor   = "0x000000";
    private String     searchColor = "0xFF0000";
    private String     borderType  = "normal";
    private String     helpFile  = "";
    private String     urlAdr    = "";

    private String     fontNode    = "serif";
    private int        fontSize    = 12;
    private boolean    isFirstTime = true;
    private Image      imageH;
    private Image      imageB;
    private Image      imageS;
    private Image      imageO;
    private Image      imageC;
    private Image      imageA;

    private Rectangle  rect; 
    private Rectangle  rectOK; 
    private Rectangle  rectCANCEL; 
    private Rectangle  rectAPLICAR; 

    private boolean    status=false;

    private HTApplet   oriApplet;

    private JFrame  vf;


  /* --- Constructor --- */

    /**
     * Constructor.
     *
     * @param model    the tree model to view
     */
    public HTView(HTModel model, Applet jApplet) {
        this.model = model; 
        draw       = new HTDraw(model, this);
	  action     = new HTAction(draw, jApplet);        
        startMouseListening();	  	 
    }

    public Insets getInsets() {
	  if (model.getEditedChild()) 
		return(new Insets(2,2,2,2));
	  else
		return(new Insets(4,4,4,4));

    }

    public void destroy() {
	//model.destroy();		
    	model  = null; // the tree model represented
	//draw.destroy();
    	draw   = null; // the drawing model
    	action = null; 	
    }

    public void setViewFrame(JFrame v) {
	vf = v;
    }

    public void setPanel(Panel p) {
		panel = p;
    }


  /* --- Paint --- */

  
    /**
     * Paint the component.
     *
     * @param g    the graphic context
     */


    public void paintBuffer(Graphics g) {  //paint
	  //super.paint(g);	
        
	  //stopMouseListening();	  

	  int x, y;

        x = 5;
        y = 5;

	  if (model.getInseredChild()) {
	  	oriApplet.reLoad(actualNode);
		model.setInseredChild(false);
	  }

	  if (model.getEditedChild()) {
		if (textField != null) {
			if (status == false) {
				panel.setVisible(true);
    				deletar.setEnabled(false);
    				inserir.setEnabled(false);
    				editar.setEnabled(false);
    				copiar.setEnabled(false);
    				colar.setEnabled(false);
    				centralizar.setEnabled(false);
    				recuperar.setEnabled(false);
    				//gravar.setEnabled(false);
    				//gravarXML.setEnabled(false);
			}

			//if  ((vf != null) && vf.isResizable()) vf.setResizable(false);
		  	if (!isSearch) requestFocus();

		}
		status = true;
	  }
	  else {
		if (status) {
			model.setTextoEdicao(textField.getText());
			status = false;
    			deletar.setEnabled(true);
    			inserir.setEnabled(true);
    			editar.setEnabled(true);
    			copiar.setEnabled(true);
    			colar.setEnabled(true);
    			centralizar.setEnabled(true);
    			recuperar.setEnabled(true);
    			//gravar.setEnabled(true);
    			//gravarXML.setEnabled(true);
		}

            panel.setVisible(false);
		//if  ((vf != null) && !vf.isResizable()) vf.setResizable(true);
	  	if (!isSearch) requestFocus();
	  }

	  if (imageB != null) {
        	g.drawImage(imageB, (getSize().width/2)-imageB.getWidth(this)/2, (getSize().height/2)-imageB.getHeight(this)/2, this);            
	  }

        draw.refreshScreenCoordinates();

        draw.drawBranches(g);

        draw.drawNodes(g);

	  if (imageH != null) {
		rect = new Rectangle(x, y, imageH.getWidth(this), imageH.getHeight(this));
        	g.drawImage(imageH, rect.x, rect.y, this);
		if (getCenterNode() != null) {
			g.drawImage(imageH, rect.x+rect.width, rect.y, this);					
		}
	  }

	  if (getIsEdited()){
		g.setColor (new Color (Color.decode(getBackColorEdit()).getRGB())); 
		g.fillRect(0, 1, getWidth(), 142);
 	  }

	  if ((imageO != null) && ( getIsEdited() ))  {
		g.setColor(new Color (Color.decode(getBackColorEdit()).getRGB())); 
		g.fillRect(100, 1, 650, 120);
		rectOK = new Rectangle(670, 1, imageO.getWidth(this), imageO.getHeight(this));
        	g.drawImage(imageO, rectOK.x, rectOK.y, this);
	  }

	  if ((imageC != null) && ( getIsEdited() ))  {
		rectCANCEL = new Rectangle(670+imageC.getWidth(this)+2, 1, imageC.getWidth(this), imageC.getHeight(this));
        	g.drawImage(imageC, rectCANCEL.x, rectCANCEL.y, this);
	  }

	  if ((imageA != null) && ( getIsEdited() ))  {
		rectAPLICAR = new Rectangle(670+(imageA.getWidth(this)*2)+4, 1, imageA.getWidth(this), imageA.getHeight(this));
        	g.drawImage(imageA, rectAPLICAR.x, rectAPLICAR.y, this);
	  }

	  if (imageS != null) {		
        	g.drawImage(imageS, getSize().width-rect.width*2 + rect.x, 10, this);				
	  }

	  if (hasHint) 
	  	draw.drawHint(g);

	  if ((isFirstTime) && (centerNode != null)) {
		isFirstTime = false;
		HTDrawNode nodeC = draw.findNode(centerNode.toLowerCase());
		if (nodeC != null) {		
			draw.translateToOrigin(nodeC);
		}    			  			
		repaint();		
	  }

	  //startMouseListening();	  

    }

    public void setHelpImage(Image i)
    {
		imageS = i;		
    }


    public void setHomeImage(Image i)
    {
		imageH = i;		
    }

    public void setConfirmarImage(Image i)
    {
		imageO = i;		
    }
    public void setCancelarImage(Image i)
    {
		imageC = i;		
    }


    public void setBackImage(Image i)
    {
		imageB = i;		
    }

    public void setAplicarImage(Image i)
    {
		imageA = i;		
    }


    public Rectangle getHomeRect() {
		return rect;
    }

    public Rectangle getOKRect() {
		return rectOK;
    }

    public Rectangle getCANCELRect() {
		return rectCANCEL;
    }

    public Rectangle getAplicarRect() {
		return rectAPLICAR;
    }



    /**
     * Set the graphics has a hint.
     *
     * @param g    the graphic context
     */


    public void setHasHint(boolean v) {
	 hasHint = v;
    }

    /**
     * Set the hint color of the nodes
     * 
     * @Param       String RGB color ("0xFFFFFF")
     * @return      void
     *              
     */

    void setHintColor(String color) 
    {
		hintColor = color;
    }

    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getHintColor() 
    {
		return hintColor;
    }

  /**
     * Set the id of the center node of the tree
     * 
     * @Param       String
     * @return      void
     *              
     */

    void setCenterNode(String nodeID) 
    {
		centerNode = nodeID;
    }

    /**
     * Returns the id of center node armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getCenterNode() 
    {
		return centerNode;
    }


    /**
     * Set the border type of the nodes
     * 
     * @Param       String
     * @return      void
     *              
     */

    void setBorderType(String bd) 
    {
		borderType = bd;
    }

    void setHelpFile(String v) {
		helpFile = v;
    }

    void setUrlAdr(String v) {
    		urlAdr = v;
    }

    String getHelpFile() {
		return helpFile;
    }

    String getUrlAdr() {
    		return urlAdr;
    }

    /**
     * Returns the border type of the node
     * 
     *
     * @return      String
     *              
     */

    String getBorderType() 
    {
		return borderType;
    }


    /**
     * Set the back color of the nodes
     * 
     * @Param       String RGB color ("0xFFFFFF")
     * @return      void
     *              
     */

    void setBackColor(String color) 
    {
		backColor = color;
    }

    void setBackColorEdit(String color) 
    {
		backColorEdit = color;
    }


    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getBackColor() 
    {
		return backColor;
    }

    String getBackColorEdit() 
    {
		return backColorEdit;
    }


    /**
     * Set the link color of the nodes
     * 
     * @Param       String RGB color ("0xFFFFFF")
     * @return      void
     *              
     */

    void setLinkColor(String color) 
    {
		linkColor = color;
    }

    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getLinkColor() 
    {
		return linkColor;
    }

    /**
     * Set the search color of the nodes
     * 
     * @Param       String RGB color ("0xFFFFFF")
     * @return      void
     *              
     */

    void setSearchColor(String color) 
    {
		searchColor = color;
    }

    /**
     * Returns the hint color armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getSearchColor() 
    {
		return searchColor;
    }

    /**
     * Set the name of the font armazened in view class
     * contains thoses given in parameters.
     *
     * @return      void
     * @param       int
     *              
     */
    
    void setFontNode(String font) 
    { 
		fontNode = font;
    }

   /**
     * Returns the name of the font armazened in view class
     * contains thoses given in parameters.
     *
     * @return      String
     *              
     */

    String getFontNode() {
		return fontNode;
    }

   /**
     * Set size of the font armazened in view class
     * contains thoses given in parameters.
     *
     * @return      void
     * @param       int             
     */

    void setFontSizeNode(int value)
    {
		fontSize = value;
    }

   /**
     * Returns the size of the font armazened in view class
     * contains thoses given in parameters.
     *
     * @return      int
     *              
     */

    int getFontSizeNode() {
		return fontSize;
    }



    /* --- Thread-safe locking --- */
  
    /**
     * Stops the listening of mouse events.
     */
    void stopMouseListening() {
        this.removeMouseListener(action);
        this.removeMouseMotionListener(action);
                this.removeKeyListener(action); 

    }
    
    /**
     * Starts the listening of mouse events.
     */
    void startMouseListening() {
        this.addMouseListener(action);
        this.addMouseMotionListener(action);
	    this.addKeyListener(action); 

    }
  
   /**
     * Set the value of text field (finds field)
     */

    void setTextField(TextField t) {
	textField = t;
    }

    void setSearchField(TextField t) {
	searchField = t;
    }

    void setCorField(TextField t) {
	corField = t;
    }
    void setCorText(TextField t) {
	corText = t;
    }
    void setHintField(TextField t) {
	hintField = t;
    }
    void setURLField(TextField t) {
	urlField = t;
    }

    void setTextLabel(Label t) {
	textLabel = t;
    }
    void setCorLabel(Label t) {
	corLabel = t;
    }
   
    void setCorTextLabel(Label t) {
	corTextLabel = t;
    }
    void setHintLabel(Label t) {
	hintLabel = t;
    }
    void setURLLabel(Label t) {
	urlLabel = t;
    }

   void setButtonGeneric(String texto, JButton button) {

		if (texto.toLowerCase().equals("remover"))
			deletar = button;
		if (texto.toLowerCase().equals("inserir"))
			inserir = button;
		if (texto.toLowerCase().equals("editar"))
			editar = button;
		if (texto.toLowerCase().equals("copiar"))
			copiar = button;
		if (texto.toLowerCase().equals("colar"))
			colar = button;
		if (texto.toLowerCase().equals("centralizar a rvore"))
			centralizar = button;
		if (texto.toLowerCase().equals("gravar rvore"))
			gravar = button;
		if (texto.toLowerCase().equals("gravar formato xml"))
			gravarXML = button;
		if (texto.toLowerCase().equals("recuperar"))
			recuperar = button;
		if (texto.toLowerCase().equals("colar arco escondido"))
			colarEscondido = button;

   }

   JButton getDeletar() {		
	return deletar;
   }
   JButton getInserir() {		
	return inserir;
   }
   JButton getEditar() {		
	return editar;
   }
   JButton getCopiar() {		
	return copiar;
   }
   JButton getColar() {		
	return colar;
   }
   JButton getCentralizar() {		
	return centralizar;
   }
   JButton getGravar() {		
	return gravar;
   }
   JButton getGravarXML() {		
	return gravarXML;
   }
   JButton getRecuperar() {		
	return recuperar;
   }
   JButton getColarEscondido() {		
	return colarEscondido;
   }


    void setCorT(Button B) { 
	B.addActionListener(action); 
	corT = B;}
    void setCorF(Button B) { 
	B.addActionListener(action); 
	corF = B;}
    void setAplicarT(Button B) { 
	B.addActionListener(action); 
	aplicarT = B;}
    void setAplicarF(Button B) { 
	B.addActionListener(action); 
	aplicarF = B;}
    void setOk(Button B) { 
	B.addActionListener(action); 
	ok = B;}
    void setCancel(Button B) { 
	B.addActionListener(action); 
	cancel = B;}

    Button getCorT() { 
	return(corT);
    }
    Button getCorF() { 
	return(corF);
    }
    Button getAplicarT() { 
	return(aplicarT);
    }
    Button getAplicarF() { 
	return(aplicarF);
    }
    Button getOk() { 
	return(ok);
    }
    Button getCancel() { 
	return(cancel);
    }
    TextField getCorFieldF() {
	return(corField);
    }
    TextField getCorTextF() {
	return(corText);
    }





   /**
     * find the string S on all node of the tree
     */

    public void searchText(HTNode node, String s) {

		String s1 = " " + s;

		boolean found = false;
		int id;

		node.setHasArc(false);
		node.setHasData(false);

		parserToken(s);

		int posH = s.toUpperCase().indexOf("_H_");
		String s2="";
		if (posH >= 0)
			s2 = s.substring(0, posH);
		else
			s2 = s;

		if (posH > 0)
			if ( (isSearch) && node.getID().toUpperCase().startsWith(s2.toUpperCase()))
				found = true;

		if (posH < 0) {
			if ( (isSearch) && searchToken(node.getName()))
				found = true;

			if ( (isSearch) &&  (!found) && searchToken(node.getHint()))
				found = true;
		}
					
		if (found) {
			node.setHasData(true);
		}

		for (Enumeration e = node.childrenNode(); e.hasMoreElements(); ) {
            	HTNode childNode = (HTNode) e.nextElement();

			childNode.setHasData(false);
			childNode.setHasArc(false);

			if (childNode != null) {
            		if (!childNode.isLeafNode()) {				
                			searchText(childNode, s);
					if ( (childNode.getHasData()) || ( childNode.getHasArc()))	
						node.setHasArc(true);					
            		}
				else {
					found = false;

					posH = s.toUpperCase().indexOf("_H_");
					s2="";
					if (posH >= 0)
						s2 = s.substring(0, posH);
					else
						s2 = s;
					if (posH > 0)
						if ( (isSearch) && childNode.getID().toUpperCase().startsWith(s2.toUpperCase())) {							
							found = true;
					}

					if (posH < 0) {
						if ( (isSearch) && searchToken(childNode.getName()))
							found = true;

						if ( (isSearch) &&  (!found) && searchToken(childNode.getHint()) )
							found = true;
					}
					
					if (found) {
						childNode.setHasData(true);
						node.setHasArc(true);						
					}
				}
													
			}
		}

	}

	

	boolean searchToken(String v) {
		String s1, s;
		int nFound=0;
		int id;
		v = v.toLowerCase();

		for(int i=0; i < nAnds;i++) {

			s = vString[i];
			s1 = " " + s.trim();

			if ( (isSearch) && ( ((id=v.indexOf(s1))>=0) || 
                   	     (v.startsWith(s)) ))			
				nFound++;
		}
		if (nFound >= nAnds) 
			return(true);
		else
			return(false);


	}

	void parserToken(String v) {
		String s="";

		int j=0;


		if (v != null) {
			for(j=0; j < v.length(); j++) if (v.charAt(j) != ' ') break;

			nAnds =0;
			for (int i=j; i < v.length();) {
		
				if (v.charAt(i) == '&') {
					if (s.length() >= 1) {
						vString[nAnds] = s;
						nAnds++;
						for(j=i+1; j < v.length(); j++) if (v.charAt(j) != ' ') break;
						i=j-1;
					}
					s = "";
				}
				else {
					s = s + v.charAt(i);
				}
				i++;								
			}
			vString[nAnds] = s;
			nAnds++;	
		}		

	}
			
   
    String getTextField() {
		String search = textField.getText();
		return search;
    }

    String getSearchField() {
		String search = searchField.getText();
		return search;
    }


    String getCorField() {
		String search = corField.getText();
		return search;
    }
    String getCorText() {
		String search = corText.getText();
		return search;
    }
    String getHintField() {
		String search = hintField.getText();
		return search;
    }
    String getURLField() {
		String search = urlField.getText();
		return search;
    }




/*

    HTNode searchPai(String Id) {
		HTModelNode nodeD = model.getRoot();
		HTNode pai;
		pai = searchPai(nodeD.getHTnode(), Id);
		return pai;
    }

    HTNode  searchPai(HTNode node, String s) {
		HTNode pai;
		boolean found = false;

		if ( node.getID().trim().equals(s.trim()))  
			return(node);

		for (Enumeration e = node.childrenNode(); e.hasMoreElements(); ) {
            	HTNode childNode = (HTNode) e.nextElement();

			if (childNode != null) {
            		if (!childNode.isLeafNode()) {				
                			pai = searchPai(childNode, s);
					return(pai);
            		}
				else {
					if (childNode.getID().trim().equals(s.trim())) 
						return(node);
				}
													
			}
		}
	}
*/

    void setActualNode(String v)
    {
		actualNode = v;
    }

    void executaSearchWord() {
		String search  = searchField.getText();

		if ((search == null) || (search.length() == 0))
			isSearch = false;
		else
			isSearch = true;
			
		HTModelNode nodeD = model.getRoot();

		searchText(nodeD.getHTnode(), search.toLowerCase());

		if (search.length() == 1)
			draw.translateToRootOrigin();

		repaint();

		searchField.requestFocus();
		
    }

   // Silvio: 29/09/2004

    void executaSearchWord(String search) {

		if ((search == null) || (search.length() == 0))
			isSearch = false;
		else
			isSearch = true;
			
		HTModelNode nodeD = model.getRoot();

		searchText(nodeD.getHTnode(), search.toLowerCase());

		repaint();

		//searchField.requestFocus();
		
    }
    // *** Silvio


    String getSearchWord() {
		String search  = searchField.getText();

		return search;
		
    }


    String getTexWord() {
		return (textField.getText());
    }

    String getCorWord() {
		return (corField.getText());
    }
    String getCorTextWord() {
		return (corText.getText());
    }
    String getHintWord() {
		return (hintField.getText());
    }
    String getURLWord() {
		return (urlField.getText());
    }

    void setTextWord(String texto) {
		textField.setText(texto);
    }

    void setSearchWord(String texto) {
		searchField.setText(texto);
    }

    void setCorWord(String texto) {
		corField.setText(texto);
		corField.setBackground(new Color (Color.decode(texto).getRGB())); 

		int r = new Color( Color.decode(texto).getRGB()).getRed(); 
		r = r - 255;
		if (r < 0) r = -r;
		int g= new Color( Color.decode(texto).getRGB()).getGreen(); 
		g = g - 255; 
		if (g < 0) g = -g;
		int b = new Color(Color.decode(texto).getRGB()).getBlue(); 
		b = b - 255;
		if (b < 0) b = -b;

		corField.setForeground(new Color (r, g, b)); 
		corField.setBackground(new Color (Color.decode(texto).getRGB())); 	

    }
    void setCorTextWord(String texto) {
		corText.setText(texto);
		int r = new Color(Color.decode(texto).getRGB()).getRed(); 
		r = r - 255;
		if (r < 0) r = -r;
		int g= new Color(Color.decode(texto).getRGB()).getGreen(); 
		g = g - 255;
		if (g < 0) g = -g;
		int b = new Color(Color.decode(texto).getRGB()).getBlue(); 
		b = b - 255;
		if (b < 0) b = -b;

		corText.setForeground(new Color (r, g, b)); 

		corText.setBackground(new Color (Color.decode(texto).getRGB())); 
    }
    void setHintWord(String texto) {
		hintField.setText(texto);
    }
    void setURLWord(String texto) {
		urlField.setText(texto);
    }

    void setApplet(HTApplet app) {
		oriApplet = app;
    }

    boolean getIsEdited() {
		return(model.getEditedChild());
    }

    public HTAction  getAction() {
		return(action);
    }

    public void undoGrava() {
		oriApplet.undoGrava();
    }

    public void grava() {
		oriApplet.grava();
    }

    public void undoRecupera() {
		if (oriApplet.hasUndo()) {
			model.setInseredChild(true);
			oriApplet.undoRecupera();
		}
    }

    public boolean getHasUndo() {		
		return(oriApplet.hasUndo());

    }

    int getNumAnds() {
	return(nAnds);
    }
  
    String getWordI(int i) {
	if (i < nAnds) 
		return (vString[i]);
	else
		return (null);
    }



  

}

